import { Injectable } from "@nestjs/common";
import { fetchAllRetailers, fetchOneRetailer, menuType, nearByRetailers } from "src/helper/dutchie-plus-query";
import { getGraphQLClient } from "src/helper/graphql-client-helper";
import { InjectRepository } from "@nestjs/typeorm";
import { RetailerServiceRepository } from "../custom-services/retailer-service/retailer-service.repository";


@Injectable()
export class DutchiePlusRetailerService {
    constructor(
      @InjectRepository(RetailerServiceRepository)
      private readonly retailerServiceRepo: RetailerServiceRepository
    ) {
    }

    async filterRetailers(data: any, search: string) {
        try {
            const searchList = [];
            const searchLowerCase = search.toLowerCase();

            for (const item of data) {
                const name = item.name.toLowerCase();
                const address = item.address.toLowerCase();
                const isMatch = name.includes(searchLowerCase) || address.includes(searchLowerCase);

                if (isMatch) {
                    searchList.push(item);
                }
            }

            return searchList;
        } catch (e) {
            return e;
        }

    }

    async checkRetailerDifference(data: any): Promise<any | Error> {
        try {
            const retailerService = await this.retailerServiceRepo.findByFilterQuery({});

            if (Object.keys(retailerService).length > 0) {
                const existingRetailerIds = retailerService.map(x => x.retailerId);
                const difference = data.retailers.filter(obj => !existingRetailerIds.includes(obj.id));
                console.log("Retailer Difference =", difference);

                const dtoObjects = difference.map(obj => ({
                    retailerId: obj.id,
                    retailerName: obj.name,
                    retailerAddress: JSON.stringify(obj.addressObject)
                }));

                await this.retailerServiceRepo.createEntities(dtoObjects);
            } else {
                const dtoObjects = data.retailers.map(obj => ({
                    retailerId: obj.id,
                    retailerName: obj.name,
                    retailerAddress: JSON.stringify(obj.addressObject)
                }));

                await this.retailerServiceRepo.createEntities(dtoObjects);
            }

        } catch (e) {
            console.log(e);
            return e;
        }
    }

    async findAll(type: string, search: string) {
        try {
            const graphQLClient = getGraphQLClient();
            const query = fetchAllRetailers();
            const data = await graphQLClient.request(query);

            // store retailers in local storage
            await this.checkRetailerDifference(data);

            // check dutchie-plus-retailer type
            if (type === "DELIVERY") {
                let deliveryList = [];
                const retailers = data.retailers;

                // Filter retailers for delivery
                for (const retailer of retailers) {
                    const delivery = retailer.hours.delivery;
                    const isAfterHoursOrdering = retailer.deliverySettings.afterHoursOrderingForDelivery;

                    if (
                      isAfterHoursOrdering &&
                      delivery.Sunday.active &&
                      delivery.Monday.active &&
                      delivery.Tuesday.active &&
                      delivery.Wednesday.active &&
                      delivery.Thursday.active &&
                      delivery.Friday.active &&
                      delivery.Saturday.active
                    ) {
                        deliveryList.push(retailer);
                    }
                }

                if (typeof search !== "undefined") {
                    return await this.filterRetailers(deliveryList, search);
                } else {
                    return deliveryList;
                }
            } else if (type === "PICKUP") {
                let pickupList = [];
                const retailers = data.retailers;

                // Filter retailers for pickup
                for (const retailer of retailers) {
                    const pickup = retailer.hours.pickup;
                    const isAfterHoursOrdering = retailer.deliverySettings.afterHoursOrderingForPickup;

                    if (
                      isAfterHoursOrdering &&
                      pickup.Sunday.active &&
                      pickup.Monday.active &&
                      pickup.Tuesday.active &&
                      pickup.Wednesday.active &&
                      pickup.Thursday.active &&
                      pickup.Friday.active &&
                      pickup.Saturday.active
                    ) {
                        pickupList.push(retailer);
                    }
                }

                if (typeof search !== "undefined") {
                    return await this.filterRetailers(pickupList, search);
                } else {
                    return pickupList;
                }
            } else {
                let allList = [];
                const retailers = data.retailers;

                // Filter all retailers with after-hours ordering for delivery or pickup
                for (const retailer of retailers) {
                    const isAfterHoursOrderingDelivery = retailer.deliverySettings.afterHoursOrderingForDelivery;
                    const isAfterHoursOrderingPickup = retailer.deliverySettings.afterHoursOrderingForPickup;

                    if (isAfterHoursOrderingDelivery || isAfterHoursOrderingPickup) {
                        const retailerService = await this.retailerServiceRepo.findOne({
                            where: {
                                retailerId: retailer.id,
                                isDeleted: false
                            }
                        });

                        retailer["contactNumber"] = retailerService?.retailerMobileNumber || "";
                        allList.push(retailer);
                    }
                }

                if (typeof search !== "undefined") {
                    return await this.filterRetailers(allList, search);
                } else {
                    return allList;
                }
            }
        } catch (e) {
            return e;
        }

    }

    async findOne(variables: any) {
        try {
            const graphQLClient = getGraphQLClient();
            const query = fetchOneRetailer();
            const data = await graphQLClient.request(query, variables);
            if (data) {
                const retailerService = await this.retailerServiceRepo.findOne({
                    where: {
                        retailerId: data.retailer.id,
                        isDeleted: false
                    }
                });
                if (retailerService) {
                    await this.retailerServiceRepo.update(retailerService.id, {
                        accessCount: retailerService.accessCount + 1
                    });
                }
                return data;
            }
        } catch (e) {
            return e;
        }
    }

    async menuType(variables: any) {
        try {
            const graphQLClient = getGraphQLClient();
            const query = menuType();
            const data = await graphQLClient.request(query, variables);
            return data["retailer"];
        } catch (e) {
            return e;
        }
    }

    async countAllRetailer() {
        try {
            const graphQLClient = getGraphQLClient();
            const query = fetchAllRetailers();
            const data = await graphQLClient.request(query);
            return Object.keys(data.retailers).length;
        } catch (e) {
            return e;
        }
    }


    async filterRetailer(variables: any, type: string, search: string) {
        try {
            let nearByRetailerData = [];
            const graphQLClient = getGraphQLClient();

            if (type === "Closest Location") {
                const query = nearByRetailers();
                const nearbyData = await graphQLClient.request(query, variables);

                for (const item of nearbyData.retailersNearLocation) {
                    const retailerId = item.retailer.id;
                    const retailerService = await this.retailerServiceRepo.findOne({
                        where: {
                            retailerId: retailerId,
                            isDeleted: false
                        }
                    });

                    if (retailerService) {
                        item.retailer["contactNumber"] = retailerService.retailerMobileNumber || "";
                    } else {
                        item.retailer["contactNumber"] = "";
                    }

                    nearByRetailerData.push(item.retailer);
                }

                if (typeof search !== "undefined") {
                    return await this.filterRetailers(nearByRetailerData, search);
                } else {
                    return nearByRetailerData;
                }
            } else {
                const query = fetchAllRetailers();
                const data = await graphQLClient.request(query);

                // Store retailers in local storage
                await this.checkRetailerDifference(data);

                let allList = [];
                const retailers = data.retailers;

                for (const retailer of retailers) {
                    const isAfterHoursOrderingDelivery = retailer.deliverySettings.afterHoursOrderingForDelivery;
                    const isAfterHoursOrderingPickup = retailer.deliverySettings.afterHoursOrderingForPickup;

                    if (isAfterHoursOrderingDelivery || isAfterHoursOrderingPickup) {
                        const retailerService = await this.retailerServiceRepo.findOne({
                            where: {
                                retailerId: retailer.id,
                                isDeleted: false
                            }
                        });

                        if (retailerService) {
                            retailer["contactNumber"] = retailerService.retailerMobileNumber || "";
                        } else {
                            retailer["contactNumber"] = "";
                        }

                        allList.push(retailer);
                    }
                }

                if (typeof search !== "undefined" && type === "ZipOrLocation") {
                    return await this.filterRetailers(allList, search);
                } else {
                    return allList;
                }
            }
        } catch (e) {
            return e;
        }
    }
}
